-- Create Quartz database for bi-platform scheduler

CREATE DATABASE IF NOT EXISTS `temperature` DEFAULT CHARACTER SET latin1;

grant all on temperature.* to 'occam'@'localhost' identified by '1qaz2wsx';

USE `temperature`;

DROP TABLE IF EXISTS TMPS_SUM_WEEK;
DROP TABLE IF EXISTS TMPS_SUM_MONTH;
DROP TABLE IF EXISTS TMPS_SUM_DAY;
DROP TABLE IF EXISTS TMPS_RAW_DAY;
DROP TABLE IF EXISTS TMPS_OPS_STATE;
DROP TABLE IF EXISTS TMPS_OPS_CFG_ALERT;
DROP TABLE IF EXISTS TMPS_OPS_CFG_SETTING;
DROP TABLE IF EXISTS TMPS_INT_LOCKS;


CREATE TABLE TMPS_SUM_DAY
  (
    SEN_TSTAMP DATETIME NOT NULL,
    SEN_NAME  VARCHAR(24) NOT NULL,
    SEN_GROUP VARCHAR(24) NOT NULL,
    DESCRIPTION VARCHAR(120) NULL,
    VALUE DECIMAL(5,2) NOT NULL,
    MAXV DECIMAL(5,2) NOT NULL,
    MINV DECIMAL(5,2) NOT NULL,
    STDV DECIMAL(5,2) NOT NULL,
    IS_ARCHIVE VARCHAR(1) NOT NULL,
    IS_ROLLUPDONE VARCHAR(1) NOT NULL,
    PRIMARY KEY (SEN_NAME,SEN_TSTAMP)
);

CREATE TABLE TMPS_SUM_WEEK
  (
    SEN_TSTAMP DATETIME NOT NULL,
    SEN_NAME  VARCHAR(24) NOT NULL,
    SEN_GROUP VARCHAR(24) NOT NULL,
    DESCRIPTION VARCHAR(120) NULL,
    VALUE DECIMAL(5,2) NOT NULL,
    MAXV DECIMAL(5,2) NOT NULL,
    MINV DECIMAL(5,2) NOT NULL,
    STDV DECIMAL(5,2) NOT NULL,
    IS_ARCHIVE VARCHAR(1) NOT NULL,
    IS_ROLLUPDONE VARCHAR(1) NOT NULL,
    PRIMARY KEY (SEN_NAME,SEN_TSTAMP)
);


CREATE TABLE TMPS_SUM_MONTH
  (
    SEN_TSTAMP DATETIME NOT NULL,
    SEN_NAME  VARCHAR(24) NOT NULL,
    SEN_GROUP VARCHAR(24) NOT NULL,
    DESCRIPTION VARCHAR(120) NULL,
    VALUE DECIMAL(5,2) NOT NULL,
    MAXV DECIMAL(5,2) NOT NULL,
    MINV DECIMAL(5,2) NOT NULL,
    STDV DECIMAL(5,2) NOT NULL,
    IS_ARCHIVE VARCHAR(1) NOT NULL,
    IS_ROLLUPDONE VARCHAR(1) NOT NULL,
    PRIMARY KEY (SEN_NAME,SEN_TSTAMP)
);

CREATE TABLE TMPS_RAW_DAY
  (
    SEN_TSTAMP DATETIME NOT NULL,
    SEN_NAME  VARCHAR(24) NOT NULL,   
    SEN_GROUP VARCHAR(24) NOT NULL,
    DESCRIPTION VARCHAR(120) NULL,
    VALUE DECIMAL(5,2) NOT NULL,
    IS_ARCHIVE VARCHAR(1) NOT NULL,
    IS_ROLLUPDONE VARCHAR(1) NOT NULL,
    PRIMARY KEY (SEN_NAME,SEN_TSTAMP)
);

CREATE TABLE TMPS_OPS_STATE
  (
    LAST_RAW DATETIME NOT NULL,
    LAST_DAY DECIMAL(5,2) NOT NULL,
    LAST_WEEK DECIMAL(5,2) NOT NULL,
    LAST_MONTH DECIMAL(5,2) NOT NULL,
    LAST_PURGE DATETIME NOT NULL
);

CREATE TABLE TMPS_OPS_CFG_ALERT
  (
    ALRT_NAME VARCHAR(80) NOT NULL,
    ALRT_GROUP VARCHAR(80) NOT NULL,
    ALRT_LEVEL DECIMAL(5,2) NOT NULL,
    ALRT_TRIGGER DECIMAL(5,2) NOT NULL,
    ALRT_RESET DECIMAL(5,2) NOT NULL,
    ALRT_SEN VARCHAR(80) NOT NULL,
    ALRT_MSG VARCHAR(80) NOT NULL,
    PRIMARY KEY (ALRT_NAME,ALRT_GROUP)
);

CREATE TABLE TMPS_OPS_CFG_SETTING
  (
    UNIT_T VARCHAR(1) NOT NULL
);

CREATE TABLE TMPS_INT_LOCKS
  (
    LOCK_NAME  VARCHAR(40) NOT NULL, 
    PRIMARY KEY (LOCK_NAME)
);

INSERT INTO TMPS_INT_LOCKS values('TMPS_OPS_STATE');
INSERT INTO TMPS_INT_LOCKS values('TMPS_OPS_CFG_ALERT');
INSERT INTO TMPS_INT_LOCKS values('TMPS_OPS_CFG_SETTING');

INSERT INTO TMPS_OPS_CFG_SETTING values('F');
commit;
