#!/usr/bin/perl

=head1 NAME

sqlupdate

=head1 SYNOPSIS

This script will perform search and replace operations on files to update 

=head1 SYNTAX

sqlupdate -f <"DR configuration file">

=head1 DESCRIPTION

This script will perform search and replace operations on the data files from an
set of files described in a configuration file. 

=head1 REVISION

11/3/1998       1.0 Created for Client. to provide a means to clone template per group
                This was a clean up of the ad hoc scripts in use already

5/30/2000       2.0 Create a "production" ready version for Client. Need to deploy templates
                and template groups for multiple customers.

=head1 AUTHOR

Mark Kirkpatrick  E<lt>F<mark.kirkpatrick@TheOccamGroup.com> E<gt>
The Occam Group

=head1 LEGAL

This script is the property of Professional Computer Services Organization 
All rights are reserved .

=cut


#---------------------------------------------------------------------------------
# USE 'S section
#---------------------------------------------------------------------------------
use FileHandle;
use POSIX;
use Time::Local;
use Getopt::Long;
use English;
#---------------------------------------------------------------------------------
# GLOBAL VARIABLES section
#---------------------------------------------------------------------------------

my($debug0) = 0;	# Inside functions debugging read_config_file

%dist_table = ();	# contains the distribution targets 
%prog_table = ();	# contains the edit lists for search and replace
local %prg_config = ();

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
sub read_config_file {
my($fh) = @_ ;
my($line1G,$valuelist);
my($tmpname,$tmpsize,$tmptype);
open(CONFIGFILE,$fh) || die "Could not open $fh : $!\n";  
while($line = <CONFIGFILE>) {
chop($line);
# print "$line\n" if $debug0;  
	CASE: {
	$line =~ /^#|^$/ && do { # skip comment lines    
	last CASE;  
	};		  	 	     

	$line =~ /START_COMMON=/i && do { 
	$dom_id = $';		
	while($line = <CONFIGFILE>) {
	chop($line);
		CASESVC: {

		$line =~ /LOGDIR=/i && do {  
			$prg_config{LOGDIR} = $';
			last CASESVC;	
		}; # LOGDIR

		$line =~ /DEBUGDIR=/i && do {  
			$prg_config{DEBUGDIR} = $';
			last CASESVC;	
		}; # DEBUGDIR

		$line =~ /LOGSIZE=/i && do {  
			$prg_config{LOGSIZE} = $';
			last CASESVC;	
		}; # LOGSIZE

		$line =~ /DEBUGSIZE=/i && do {  
			$prg_config{DEBUGSIZE} = $';
			last CASESVC;	
		}; # DEBUGSIZE

                $line =~ /DIST_MASTER=/i && do {
                        $prg_config{DEBUGSIZE} = $';
                        last CASESVC;
                }; # DEBUGSIZE

                $line =~ /DEBUGSIZE=/i && do {
                        $prg_config{DEBUGSIZE} = $';
                        last CASESVC;
                }; # DEBUGSIZE

		$line =~ /END_COMMON/i && do {
			last CASE;	
		}; # END_COMMON_
		} # end of CASESVC
	} # end of sub while
	}; # START_COMMON

	$line =~ /START_DIST_TARGET=\s*/i && do { 
	$dom_id = $';		
	$valuelist = 0;
	while($line = <CONFIGFILE>) {
	chop($line);
		CASEDOMPROG: {

		$line =~ /DIST_MASTER=/i && do {  
			$dist_table{$dom_id}{DIST_MASTER} = $';
			last CASEDOMPROG;	
		}; #DIST_MASTER 

		$line =~ /HOST=/i && do {  
			$dist_table{$dom_id}{HOST}[$valuelist] = $';
			print "$valuelist:: $dist_table{$dom_id}{HOST}[$valuelist]\n" if $debug0;  
			$valuelist++;
			last CASEDOMPROG;	
		}; #HOST 

		$line =~ /END_DIST_TARGET/i && do {
			last CASE;	
		}; #END_DIST_TARGET 
		} # end of CASEDOMPROG
	} # end of sub while
	}; #START_DIST_TARGET 

	$line =~ /START_EDIT_LIST=\s*/i && do { 
	$dom_id = $';		
	$valuelist = 0;
	while($line = <CONFIGFILE>) {
	chop($line);
		CASEEDL: {

		$line =~ /HOST=/i && do {  
			$prog_table{$dom_id}{HOST}[$valuelist] = $';
			print "$valuelist:: $prog_table{$dom_id}{HOST}[$valuelist]\n" if $debug0;  
			$valuelist++;
			last CASEEDL;	
		}; #HOST 

		$line =~ /END_EDIT_LIST/i && do {
			last CASE;	
		}; #END_EDIT_LIST 
		} # end of CASEEDL 
	} # end of sub while
	}; #START_EDIT_LIST 


	}; # end of CASE BLOCK statment
	} # End of while loop to read file  
	close(CONFIGFILE);
};
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
print<<EOMESSAGE;
*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*

This script will update / verify steps required for
hosts in a DR environment.

1. Collect existing license data from dr systems and save it
2. Distribute updated licenses and run Updatelicense.sh for ETS hosts
3. Set permissions to weblog81:bea for license files

*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
EOMESSAGE

#---------------------------------------------------------------------------------
# MAIN PROGRAM section
#---------------------------------------------------------------------------------
my ($ready,$timeleft,$rbits_out,$rbits,$wbits_out,$wbits,$status,$listen_socket,$child,$Debug );
local ( $starttime,$stoptime,$elapsetime );
my ($t_sec,$t_min,$t_hour,$t_day,$t_mon,$t_year,$t_wday,$t_yday,$t_isdst);

# parse the command line options
@optl = ( "f=s","debug" );
$usagestr = "Usage $0 -f <Config file name> [-debug]\n";
die "$usagestr" unless GetOptions @optl;
die "You must provide a value for file name \(-f\) \n$usagestr" unless $opt_f;
# die "You must provide a value for bef \(-bef\) \nUsage $0 -dir <Config download directory> [-debug]\n" unless $opt_dir;

select(STDOUT); $|=1;

# Initialize some default variables
$0 =~ s/.*\///;
my($config_file) = '$0.cfg';
$config_file = $opt_f if $opt_f;
$prg_config{LOGNAME} = "$0";
$prg_config{LOGINT} = "1d";
$prg_config{LOGDIR} = "/corp/logs";
$prg_config{DEBUGDIR} = "/corp/logs";
$prg_config{ROLLBACK} = 360;
$prg_config{LOGSIZE} = 50000 ;
$prg_config{DEBUGSIZE} = 50000 ;
$prg_config{BLOCKTIME} = 10 ;
$prg_config{PROCESSNAME} = $0 ; # reset if you want a special name

$0=$prg_config{PROCESSNAME};
$PRG_BASE = $prg_config{LOGNAME};
# Calculate the number of seconds to wait based on the LOGINT setting.
( $intnum, $inttype ) = ( $prg_config{LOGINT} =~ /(\d+)(\w)/i );
$intsize = 1 if ( uc($inttype) eq 'S' );
$intsize = 60 if ( uc($inttype) eq 'M' );
$intsize = 3600 if ( uc($inttype) eq 'H' );
$intsize = 86400 if ( uc($inttype) eq 'D' );
$intsize = ( 86400 * 7 ) if ( uc($inttype) eq 'W' );

&read_config_file($config_file);

MAIN: {

# Main is in a block though it does not loop it easily could.
# Get and format the time
$starttime = time;
$epochtime = $starttime;
($t_sec,$t_min,$t_hour,$t_day,$t_mon,$t_year,$t_wday,$t_yday,$t_isdst) = localtime($starttime);
$newtime = $starttime - ( $t_isdst * 3600 );
($t_sec,$t_min,$t_hour,$t_day,$t_mon,$t_year,$t_wday,$t_yday,$t_isdst) = localtime($newtime);
$t_mon++;

$t_sec = '0' . $t_sec if ( length($t_sec) == 1 );
$t_min = '0' . $t_min if ( length($t_min) == 1 );
$t_hour = '0' . $t_hour if ( length($t_hour) == 1 );
$t_day = '0' . $t_day if ( length($t_day) == 1 );
$t_mon = '0' . $t_mon if ( length($t_mon) == 1 );
$t_year = '0' . $t_year if ( length($t_year) == 1 );

$start_dt_stamp = "_" . $t_mon . $t_day . $t_year . "_" . $t_hour . $t_min . $t_sec;
$rolltime = $epochtime + ( $intnum * $intsize );

#	for $edit (@{ $prog_table{$dom}{HOST} })
#	 {
	  # print "$dom: ->$edit<-\n";
#	  ($tmphost,$tmpfile,$tmpfrom,$tmpto) = split(/~~/,$edit);
	  $tmpfile = $opt_f;
          open DFIN, "<${tmpfile}";
          @bigstring = <DFIN>;
          close DFIN;
          $"='';
          $bigstring = "@bigstring";
          $"= ' ';

=sample1 Logfile without timestamps

Requesting temperatures... DONE
Device Address: 28 54 D6 5E 2 0 0 AC Temp C: 72.50 Temp F: 162.50
Device Address: 28 7C 2 5F 2 0 0 75 Temp C: 70.50 Temp F: 158.90
Device Address: 28 C9 D6 5E 2 0 0 57 Temp C: 41.00 Temp F: 105.80
Device Address: 28 F9 CB 5E 2 0 0 2A Temp C: 68.00 Temp F: 154.40
SourceFeed higher than LoadFeed set relay flag to HIGH/ON 162/110
Setting Relay pin HIGH Run Sec: 301
log:SourceFeed:F:162.50
log:SourceReturn:F:158.90
log:LoadFeed:F:105.80
log:LoadReturn:F:154.40

=cut

# Sample:
# CREATE UNIQUE INDEX "CLAIM"."LINEITEM_PK" ON "CLAIM"."LINEITEM" ("LINEITEM_ID") 
open DFOUT, ">${tmpfile}.sql";
while ( $bigstring =~ /\nD(\d{2})(\d{2})(\d{4})_T(\d{2})(\d{2})(\d{2})\s+.*?(?=\nD(\d{2})(\d{2})(\d{4})_T)/sg )
{
	  # Get a single timestamped line
	  $tmpstr = $MATCH;
	  if ( $tmpstr =~ /Setting Relay/sg)
	  {
		print "Relay>>$tmpstr<<\n";
		$tmpstr =~ s/LOGGING(.*)NOLOGGING/LOGGING\1/sg;
		print "Relay>>$tmpstr<<\n";
	  }
	  # Parse the "log" line
	  ($ts_m,$ts_d,$ts_y,$ts_H,$ts_M,$ts_S,$tname,$tunit,$tval) = ( $tmpstr =~ /D(\d{2})(\d{2})(\d{4})_T(\d{2})(\d{2})(\d{2})\s+log:(\w+):(F|C):(\d+\.\d+)/sg );
	  # If we messed up.
	  print "-->$tname was  $tvalue<--\n"; 
	  if ( $tname eq '' ) 
	  { print ">>>$tmpstr<<<\n";
	  }
	  # Make a pretty string of the timestamp ( We will fix this later into a real datatype for datetime )
	  $data_ts = "${ts_y}-${ts_m}-${ts_d} ${ts_H}:${ts_M}:${ts_S}";
	  if ( $tname =~ /Source/sg)
          {
	    print DFOUT "insert into TMPS_RAW_DAY (SEN_TSTAMP,SEN_NAME,SEN_GROUP,DESCRIPTION,VALUE,IS_ARCHIVE,IS_ROLLUPDONE) values (\'$data_ts\',\'$tname\',\'Source\',\'Sensor measured\',$tval,\'N\',\'N\' )\;\n";
          } elsif ( $tname =~ /Load/sg)
	  {
	    print DFOUT "insert into TMPS_RAW_DAY (SEN_TSTAMP,SEN_NAME,SEN_GROUP,DESCRIPTION,VALUE,IS_ARCHIVE,IS_ROLLUPDONE) values (\'$data_ts\',\'$tname\',\'Load\',\'Sensor measured\',$tval,\'N\',\'N\' )\;\n";
	  }
	  # print DFOUT "DROP INDEX $indexname\;\n";
	  # print DFOUT $tmpstr
	 
 }
          flush DFOUT;
          close DFOUT;

exit;

}
